import enum
import mimetypes
import re
import smtplib
from email.mime.application import MIMEApplication
from email.mime.audio import MIMEAudio
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formatdate
from pathlib import Path
from typing import BinaryIO, List, TextIO, Union


def _helperOutputFileCheck(filename: str) -> bool:
    try:
        if Path(filename).is_file():
            try:
                fileSize: int = Path(filename).stat().st_size
            except Exception as error:
                raise error
            if fileSize != 0:
                raise OSError(
                    "outputFile is not empty, please empty or delete it and try again"
                )
        else:
            try:
                Path(filename).touch()
            except Exception as error:
                raise PermissionError(f"No access to create the new file - {error}")
    except Exception as error:
        raise PermissionError(
            f"No access to file location to check if file exists - {error}"
        )
    return True


class _EncType(enum.Enum):
    """
    Enumerations of TLS and SSL, for encryption logic in Emailee
    """

    SSL = 1
    TLS = 2


class Emailee:
    """
    A robust SMTP email solution for Python.
    Zero external dependencies (unless testing).
    Allows sending via SSL or TLS, attachments, reply-to and to/cc/bcc.
    Emails are built with the below structure -

    multipart/mixed
        ├─── multipart/related
        │    └── multipart/alternative
        │        ├─── text/plain
        │        └─── text/html
        ├─── application/text - embedded attachment
        ├─── application/image - embedded attachment
        ├─── application/audio - embedded attachment
        └─── application/pdf etc. - other application attachment

    See https://github.com/mattwalshdev/emailee for additional documentation

    Methods
    -------
        sender(sender: str, replyTo: str) - email (required), email
        subject(subject: str) - text
        msgContent(msgText: str, msgHTML: str) - raw text, html text
        sendTo(to: List[str], cc: List[str], bcc: List[str], ignoreErrors: bool) -
            List of emails in each, one required. Set ignoreErrors True if you
            want it to still send emails if there is 1+ invalid emails listed
        attachmentFiles(attachmentFiles: List[str]) - List of full and/or
            relative paths to files
        server(smtpServer: str, port: int, SSLTLS: str, authUsername: str,
            authPassword: str, timeout: int) - servername (required),
            port no, encryption type, credentials,
            timeout for server connection timeout in seconds, default 30 seconds,
            increase if on slow connection
        ready() - will return True if all required fields are entered and valid,
            does not guarantee emails actually exist
        send(outputFile: str) - attempt to send the email, optional addition of an
            outputFile to save metadata to. OutputFile is mandatory when sending emails in either async mode

    Example
    -------
    import emailee

    my_email = emailee.Emailee()
    my_email.sender("john.smith@fakeemail.com")
    my_email.subject("Test email")
    my_email.msgContent("raw text, poorly formatted")
    my_email.sendTo(["jill.smith@fakeemail.com"])
    my_email.server("smtp.fakeemail.com")

    if my_email.ready():
        my_email.send()
    """

    _EMAIL_REGEX = re.compile(r"(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)")

    def __init__(self) -> None:
        self._sender: str = ""
        self._replyTo: str = ""
        self._subject: str = ""
        self._msgText: str = ""
        self._msgHTML: str = ""
        self._to: List[str] = []
        self._cc: List[str] = []
        self._bcc: List[str] = []
        self._attachmentFiles: List[str] = []
        self._smtpServer: str = ""
        self._port: int = 0
        self._SSLTLS: Union[_EncType, None] = None
        self._authUsername: str = ""
        self._authPassword: str = ""
        self._timeout: int = 30
        self._outputFile: str = ""
        self._outputFileReady: bool = False

    def __repr__(self) -> str:
        outputDict = {
            "sender": self._sender,
            "replyTo": self._replyTo,
            "subject": self._subject,
            "msgText": self._msgText,
            "msgHTML": self._msgHTML,
            "to": self._to,
            "cc": self._cc,
            "bcc": self._bcc,
            "attachmentFiles": self._attachmentFiles,
            "smtpServer": self._smtpServer,
            "port": self._port,
            "SSLTLS": self._SSLTLS,
        }
        return str(outputDict)

    def __str__(self) -> str:
        outputDict = {
            "sender": self._sender,
            "replyTo": self._replyTo,
            "subject": self._subject,
            "to": self._to,
            "cc": self._cc,
            "bcc": self._bcc,
        }
        return str(outputDict)

    def _addressCheck(
        self, emailList: List[str], type: str, ignoreErrors: bool
    ) -> List[str]:
        self._tempEmailList = []
        for email in emailList:
            if not isinstance(email, str):
                raise TypeError(f"{type} email not a string - {email}")

            if not Emailee._EMAIL_REGEX.fullmatch(email):
                if not ignoreErrors:
                    raise ValueError(f"Invalid {type} email - {email}")
                else:
                    continue
            if ignoreErrors:
                self._tempEmailList.append(email)
        if ignoreErrors:
            return self._tempEmailList
        return emailList

    def sender(self, sender: str, replyTo: str = "") -> None:
        if not isinstance(sender, str):
            raise TypeError("sender not a string")

        self._sender = self._addressCheck([sender], "sender", ignoreErrors=False)[0]

        if replyTo:
            if not isinstance(replyTo, str):
                raise TypeError("replyTo not a string")

            self._replyTo = self._addressCheck(
                [replyTo], "replyTo", ignoreErrors=False
            )[0]

    def subject(self, subject: str) -> None:
        if not isinstance(subject, str):
            raise TypeError("subject not a string")

        if len(subject) > 255:
            raise ValueError("subject string too long")
        self._subject = subject

    def msgContent(self, msgText: str = "", msgHTML: str = "") -> None:
        if not isinstance(msgText, str):
            raise TypeError("msgText not a string")

        if not isinstance(msgHTML, str):
            raise TypeError("msgHTML not a string")

        self._msgText = msgText
        self._msgHTML = msgHTML

    def sendTo(
        self,
        to: List[str] = [],
        cc: List[str] = [],
        bcc: List[str] = [],
        ignoreErrors: bool = False,
    ) -> None:
        if not to and not cc and not bcc:
            raise ValueError("No email addresses to send to are defined")

        if (
            not isinstance(to, list)
            or not isinstance(cc, list)
            or not isinstance(bcc, list)
        ):
            raise TypeError("sendTo items are not in list format")

        if not isinstance(ignoreErrors, bool):
            raise TypeError("sendTo ignoreErrors is not a bool")

        self._to = self._addressCheck(to, "to", ignoreErrors)
        self._cc = self._addressCheck(cc, "cc", ignoreErrors)
        self._bcc = self._addressCheck(bcc, "bcc", ignoreErrors)

        if not self._to and not self._cc and not self._bcc:
            raise ValueError("No email addresses to send to are properly defined")

    def attachmentFiles(self, attachmentFiles: List[str] = []) -> None:
        if not isinstance(attachmentFiles, list):
            raise TypeError("attachmentFiles not in list format")

        for file in attachmentFiles:
            if not isinstance(file, str):
                raise TypeError(f"file name listed not in string format - {file}")

            if not Path(file).is_file():
                raise ValueError(
                    f"Attachment listed is not a file or not accessible - {file}"
                )

        self._attachmentFiles = attachmentFiles

    def server(
        self,
        smtpServer: str,
        *,
        port: int = 0,
        SSLTLS: str = "",
        authUsername: str = "",
        authPassword: str = "",
        timeout: int = 30,
    ) -> None:
        if not isinstance(smtpServer, str):
            raise TypeError("smtpServer name not in string format")

        if not isinstance(port, int):
            raise TypeError("smtpServer port not in integer format")

        if not isinstance(SSLTLS, str):
            raise TypeError("smtpServer SSLTLS option not in string format")

        if not isinstance(authUsername, str):
            raise TypeError("smtpServer auth username not in string format")

        if not isinstance(authPassword, str):
            raise TypeError("smtpServer auth password not in string format")

        if not isinstance(timeout, int):
            raise TypeError("timeout server option not in integer format")

        if port < 0 or port > 65535:
            raise ValueError("Port is an invalid number")

        if SSLTLS not in _EncType.__members__ and SSLTLS != "":
            raise ValueError("SSLTLS option is invalid")

        if timeout <= 0:
            raise ValueError("Server timeout period is an invalid number")

        self._smtpServer = smtpServer

        if port > 0:
            self._port = port
        elif SSLTLS == "TLS":
            self._port = 587
        elif SSLTLS == "SSL":
            self._port = 465
        else:
            self._port = 25

        if SSLTLS:
            self._SSLTLS = _EncType[f"{SSLTLS}"]
        elif port == 587:
            self._SSLTLS = _EncType.TLS
        elif port == 465:
            self._SSLTLS = _EncType.SSL

        if authUsername and authPassword:
            self._authUsername = authUsername
            self._authPassword = authPassword
        elif authPassword:
            self._authUsername = self._sender
            self._authPassword = authPassword

        self._timeout = timeout

    def ready(self) -> bool:
        if self._authPassword and not self._authUsername and self._sender:
            # this may happen if server methods are invoked before
            #  the sender method AND the person has not added an
            #  authUsername BUT has added an authPassword
            self._authUsername = self._sender

        if (
            self._sender
            and (self._to or self._cc or self._bcc)
            and self._smtpServer
            and self._port
        ):
            # minimum requirements in place to send an email
            return True
        else:
            return False

    def _generate(self) -> str:
        message = MIMEMultipart("mixed")

        message["to"] = ", ".join(self._to) if self._to else ""

        message["cc"] = ", ".join(self._cc) if self._cc else ""

        message["from"] = self._sender
        message["subject"] = self._subject
        message["date"] = formatdate(localtime=True)

        if self._replyTo:
            message.add_header("reply-to", self._replyTo)

        messageRelated = MIMEMultipart("related")
        messageAlternative = MIMEMultipart("alternative")

        if self._msgText:
            messageAlternative.attach(MIMEText(self._msgText, "plain"))
        if self._msgHTML:
            messageAlternative.attach(MIMEText(self._msgHTML, "html"))

        messageRelated.attach(messageAlternative)

        message.attach(messageRelated)

        for attFile in self._attachmentFiles or []:
            contentType, encoding = mimetypes.guess_type(attFile)

            if contentType is None or encoding is not None:
                contentType = "application/octet-stream"
            mainType, subType = contentType.split("/", 1)

            attachment: Union[MIMEText, MIMEImage, MIMEAudio, MIMEApplication]
            fp: Union[TextIO, BinaryIO]

            if mainType == "text":
                with open(attFile, "r") as fp:
                    attachment = MIMEText(fp.read(), _subtype=subType)
            else:
                with open(attFile, "rb") as fp:
                    if mainType == "image":
                        attachment = MIMEImage(fp.read(), _subtype=subType)
                    elif mainType == "audio":
                        attachment = MIMEAudio(fp.read(), _subtype=subType)
                    else:
                        attachment = MIMEApplication(fp.read(), _subtype=subType)

            filename = Path(attFile).stem
            attachment.add_header(
                "Content-Disposition", "attachment; filename=" + filename
            )
            message.attach(attachment)

        return message.as_string()

    def send(self, outputFile: str = "") -> bool:
        if self._authPassword and not self._authUsername and self._sender:
            # this may happen if server methods are invoked before
            #  the sender method AND the person has not added an
            #  authUsername BUT has added an authPassword
            self._authUsername = self._sender

        if not isinstance(outputFile, str):
            raise TypeError("outputFile path not in string format")

        # run checks against the output file, unless it's already been run
        #  during this run, this is for async batch jobs of emails
        #  where send is called repeatedly
        if outputFile:
            if not self._outputFileReady:
                self._outputFileReady = _helperOutputFileCheck(outputFile)
            self._outputFile = outputFile

        try:
            generatedEmail: str = self._generate()
        except Exception as error:
            raise (error)

        try:
            smtp: object

            if self._SSLTLS == _EncType.SSL:
                try:
                    smtp = smtplib.SMTP_SSL(
                        self._smtpServer, self._port, timeout=self._timeout
                    )
                except Exception as error:
                    raise ValueError(error)
            else:
                try:
                    smtp = smtplib.SMTP(
                        self._smtpServer, self._port, timeout=self._timeout
                    )
                except Exception as error:
                    raise ValueError(error)

            smtp.ehlo()

            if self._SSLTLS == _EncType.TLS:
                smtp.starttls()
                smtp.ehlo()
        except Exception as error:
            raise error
        if self._authUsername and self._authPassword:
            try:
                smtp.login(self._authUsername, self._authPassword)
            except Exception as error:
                raise ValueError(error)

        try:
            smtp.sendmail(self._sender, self._to + self._cc + self._bcc, generatedEmail)
        except Exception as error:
            raise ValueError(error)

        try:
            smtp.close()
            if self._outputFile:
                with open(self._outputFile, "a") as file:
                    file.write(self.__str__() + "\n")
            return True
        except Exception as error:
            raise (error)



import multiprocessing
import threading
import time
from typing import Any, Dict, List, Union

from .emailee import Emailee, _helperOutputFileCheck


class _SendAsync:
    """
    No point having a great SMTP library without the ability to send asynchronous mail!
    You can roll your own using the Emailee class but all the hard work is already done here.
    Uses either Threaded or Multiprocessing libraries:
        Threaded - Create x amount of threads on same CPU core
        Multiprocessing - Create processes utilising all CPU cores
    See docstrings for AsyncThreads and AsyncMP on how to implement.

    Parameters
    -------
        mailList -
            [
                {
                    'sender': sender email,
                    'replyTo': reply to email, optional
                    'subject': email subject, optional
                    'msgText': raw text message, optional
                    'msgHTML': HTML formatted message, optional
                    'to': list of to emails, optional
                    'cc': list of cc emails, optional
                    'bcc': list of bcc emails, optional
                    'ignoreErrors': will continue to send the email, skipping invalid to/cc/bcc mail items, optional
                    'attachmentFiles': list of attachment file paths, optional
                },
                {...},
            ]
        serverDict -
            {
                'smtpServer': SMTP server,
                'port': port number, optional
                'SSLTLS': SSL or TLS encryption, optional
                'authUsername': authenticated username, optional
                'authPassword': authenticated password, optional
            }
        outputFile: str - empty or non-existant text file to write successful sent email metadata to.
            Similar to AsyncThreads/AsyncMP.emailReport(), but the outputFile is written to during the email
            processing, so if there is an exception which crashes the mail run you will have a list of what successfully
            sent. Helpful to check against mail send throttling issues, which you may not notice otherwise.
        waitTime: int or float - time in seconds between each mail item sent, useful if your sending too
             many emails, too quickly utilising services like gmail, who will stop sending emails if it looks like spam
    """

    def __init__(
        self,
        mailList: List[Dict[str, Any]],
        serverDict: Dict[str, Any],
        outputFile: str,
        waitTime: Union[int, float] = 0,
    ) -> None:
        if not isinstance(mailList, list):
            raise TypeError("Email items not valid type")

        if not isinstance(serverDict, dict):
            raise TypeError("Server items not valid type")

        if not isinstance(waitTime, int) and not isinstance(waitTime, float):
            raise TypeError("Email wait time not valid number")

        if waitTime < 0:
            raise ValueError("waitTime cannot be a negative number")

        self._mailList: List[Dict[str, str]] = mailList
        self._serverDict: Dict[str, str] = serverDict
        self._waitTime: Union[int, float] = waitTime
        self._outputFileReady: bool = False

        self.emailReport: List[Dict[str, Any]] = []

        self._processQueue: multiprocessing.Queue = multiprocessing.Queue()

        self._outputFileReady = _helperOutputFileCheck(outputFile)
        self._outputFile: str = outputFile

        self._outputFileReady

    def _runTest(self):
        # run initial mail item procedurally to check server works
        try:
            _sendMailFunc(
                self._mailList[0],
                self._serverDict,
                self._processQueue,
                self._outputFile,
                self._outputFileReady,
            )
            self.emailReport.append(self._processQueue.get())
        except Exception as error:
            raise ValueError(error)


def _sendMailFunc(
    mailItem: Dict,
    mailServer: Dict,
    processQueue: multiprocessing.Queue,
    outputFile: str,
    outputFileReady: bool,
) -> None:
    """
    Helper function that sends each asynchronous mail item
    when using AsyncThreads or AsyncMP

    Parameters
    -------
        mailItem - Individual mail dict from _SendAsync.self._mailList
        mailServer - Server settings in dict from _SendAsync.self._serverDict
        processQueue - Queue object that sending results are added to
        outputFile - Text file append successful sends metadata to
    """

    mailItem.setdefault("sender", "")
    mailItem.setdefault("replyTo", "")
    mailItem.setdefault("subject", "")
    mailItem.setdefault("msgText", "")
    mailItem.setdefault("msgHTML", "")
    mailItem.setdefault("to", [])
    mailItem.setdefault("cc", [])
    mailItem.setdefault("bcc", [])
    mailItem.setdefault("ignoreErrors", False)
    mailItem.setdefault("attachmentFiles", [])

    mailServer.setdefault("smtpServer", [])
    mailServer.setdefault("port", 0)
    mailServer.setdefault("SSLTLS", None)
    mailServer.setdefault("authUsername", "")
    mailServer.setdefault("authPassword", "")
    mailServer.setdefault("timeout", 30)

    mail = Emailee()
    # override the class attribute as we've already checked the output
    #  file is valid for all async email sends
    mail._outputFileReady = outputFileReady
    mail.sender(sender=mailItem["sender"], replyTo=mailItem["replyTo"])
    mail.subject(mailItem["subject"])
    mail.msgContent(msgText=mailItem["msgText"], msgHTML=mailItem["msgHTML"])
    mail.sendTo(
        to=mailItem["to"],
        cc=mailItem["cc"],
        bcc=mailItem["bcc"],
        ignoreErrors=mailItem["ignoreErrors"],
    )
    mail.attachmentFiles(mailItem["attachmentFiles"])

    mail.server(
        smtpServer=mailServer["smtpServer"],
        port=mailServer["port"],
        SSLTLS=mailServer["SSLTLS"],
        authUsername=mailServer["authUsername"],
        authPassword=mailServer["authPassword"],
        timeout=mailServer["timeout"],
    )
    try:
        if mail.ready():
            mail.send(outputFile)
            processQueue.put(
                {
                    "sender": mailItem["sender"],
                    "replyTo": mailItem["replyTo"],
                    "subject": mailItem["subject"],
                    "to": mailItem["to"],
                    "cc": mailItem["cc"],
                    "bcc": mailItem["bcc"],
                }
            )
    except Exception as error:
        raise ValueError(error)


class _myThread(threading.Thread):
    """
    Class called by AsyncThreads to generate a new thread

    Parameters
    -------
        mailItem - Individual mail dict from _SendAsync.self._mailList
        serverDict - Server settings in dict from _SendAsync.self._serverDict
        poolSema - Pool BoundedSemaphore for threading
        processQueue - Queue object that sending results are added to
    """

    def __init__(
        self,
        mailItem: Dict,
        serverDict: Dict,
        poolSema: threading.BoundedSemaphore,
        processQueue: multiprocessing.Queue,
        outputFile: str,
        outputFileReady: bool,
    ) -> None:
        threading.Thread.__init__(self)

        self._mailItem: Dict = mailItem
        self._serverDict: Dict = serverDict
        self._poolSema = poolSema
        self._processQueue = processQueue
        self._outputFile: str = outputFile
        self._outputFileReady: bool = outputFileReady

    def run(self):
        self._poolSema.acquire()
        try:
            _sendMailFunc(
                self._mailItem,
                self._serverDict,
                self._processQueue,
                self._outputFile,
                self._outputFileReady,
            )
        except Exception as error:
            raise error
        finally:
            self._poolSema.release()


class AsyncThreads(_SendAsync):
    """
    Class to call to send emails asyncronously using threads.
    Threads are bound to the CPU that the core program sits on
    but can be rate limited if you don't want to max out your hardware.
    AsyncThreads.emailReport will return a list of all sent emails, useful for logging

    See https://github.com/mattwalshdev/emailee for additional documentation

    Parameters
    -------
        mailList - List of all mail items, sent to _SendAsync
        serverDict - Server settings in dict, sent to _SendAsync
        maxThreads - Maximum number of concurrent threads allowed, optional, default 10
        outputFile - Empty text file to store successfuly sends, sent to _SendAsync
        waitTime - Time in seconds before each email item, sent to _SendAsync

    Example
    -------
    import emailee

    emails = [{...}]
    server = {...}

    emails = emailee.AsyncThreads(emails, server, outputFile='output.txt', waitTime=0.250)
    print(emails.emailReport)
    """

    def __init__(
        self,
        mailList: List[Dict[str, Any]],
        serverDict: Dict[str, Any],
        outputFile: str,
        maxThreads: int = 10,
        waitTime: Union[int, float] = 0,
    ) -> None:
        _SendAsync.__init__(self, mailList, serverDict, outputFile, waitTime)

        if not isinstance(maxThreads, int):
            raise TypeError("maxThread is not valid int")

        if maxThreads <= 0:
            raise ValueError("maxThreads must be a number greater than 0")
        self._maxThreads = maxThreads

        self._active: List = []

        self.run()

    def run(self) -> List:
        self._runTest()
        if len(self._mailList) == 1:
            return self.emailReport

        self._poolSema = threading.BoundedSemaphore(value=self._maxThreads)

        for mailItem in self._mailList[1:]:
            newThread = _myThread(
                mailItem,
                self._serverDict,
                self._poolSema,
                self._processQueue,
                self._outputFile,
                self._outputFileReady,
            )
            newThread.start()
            self._active.append(newThread)
            time.sleep(self._waitTime)

        # wait for all items to complete before exiting
        for jobs in self._active:
            jobs.join()

        while self._processQueue.qsize():
            self.emailReport.append(self._processQueue.get())

        return self.emailReport


class AsyncMP(_SendAsync):
    """
    Class to call to send emails asyncronously using multiprocessing.
    Processes are able to utilise all CPU cores, but take up more memory
    due to isolated binaries copied for each process. Depending on work load
    you may utilise 100% of your CPU cycles running this.
    AsyncMP.emailReport will return a list of all sent emails, useful for logging

    See https://github.com/mattwalshdev/emailee for additional documentation

    Parameters
    -------
        mailList - List of all mail items, sent to _SendAsync
        serverDict - Server settings in dict, sent to _SendAsync
        outputFile - Empty text file to store successfuly sends, sent to _SendAsync
        waitTime - Time in seconds before each email item, sent to _SendAsync

    Example
    -------
    import emailee

    emails = [{...}]
    server = {...}

    emails = emailee.AsyncMP(emails, server, outputFile='output.txt', waitTime=0.250)
    print(emails.emailReport)
    """

    def __init__(
        self,
        mailList: List[Dict[str, Any]],
        serverDict: Dict[str, Any],
        outputFile: str,
        waitTime: Union[int, float] = 0,
    ) -> None:
        _SendAsync.__init__(self, mailList, serverDict, outputFile, waitTime)

        self._active: List = []

        self.run()

    def run(self) -> List:
        self._runTest()
        if len(self._mailList) == 1:
            return self.emailReport

        for mailItem in self._mailList[1:]:
            newProcess = multiprocessing.Process(
                target=_sendMailFunc,
                args=(
                    mailItem,
                    self._serverDict,
                    self._processQueue,
                    self._outputFile,
                    self._outputFileReady,
                ),
            )
            newProcess.start()
            self._active.append(newProcess)
            time.sleep(self._waitTime)

        # wait for all items to complete before exiting
        for jobs in self._active:
            jobs.join()

        while not self._processQueue.empty():
            self.emailReport.append(self._processQueue.get())

        return self.emailReport



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['emailee']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'emailee',
    'version': '1.0.0',
    'description': 'A robust smtp email send tool, with asyncronous send capability',
    'long_description': '# emailee\n\n[![PyPI Version][pypi-image]][pypi-url]\n[![Build Status][build-image]][build-url]\n[![Code Coverage][coverage-image]][coverage-url]\n[![Code Quality][quality-image]][quality-url]\n[![Code Factor][codefactor-image]][codefactor-url]\n[![versions][versions-image]][versions-url]\n[![license][license-image]][license-url]\n\n<!-- Badges -->\n\n[pypi-image]: https://img.shields.io/pypi/v/emailee\n[pypi-url]: https://pypi.org/project/emailee/\n\n[build-image]: https://github.com/mattwalshdev/emailee/actions/workflows/build.yml/badge.svg\n[build-url]: https://github.com/mattwalshdev/emailee/actions/workflows/build.yml\n\n[coverage-image]: https://codecov.io/gh/mattwalshdev/emailee/branch/main/graph/badge.svg\n[coverage-url]: https://codecov.io/gh/mattwalshdev/emailee\n\n[quality-image]: https://api.codeclimate.com/v1/badges/0360d77f65da3d985d03/maintainability\n[quality-url]: https://codeclimate.com/github/mattwalshdev/emailee\n\n[codefactor-image]: https://www.codefactor.io/repository/github/mattwalshdev/emailee/badge\n[codefactor-url]: https://www.codefactor.io/repository/github/mattwalshdev/emailee\n\n[versions-image]: https://img.shields.io/pypi/pyversions/emailee.svg\n[versions-url]: https://github.com/mattwalshdev/emailee\n\n[license-image]: https://img.shields.io/github/license/mattwalshdev/emailee.svg\n[license-url]: https://github.com/mattwalshdev/emailee/blob/main/LICENSE\n\nA robust SMTP mailer library, with async mail send features built-in using Threads and Multiprocessing. Zero PyPI (production) dependencies.\n\n## Installation\n\nInstall with `pip install emailee`.\n\n## Some quick examples\n\nSend a simple text email\n\n```Python\nimport emailee\n\nmy_email = emailee.Emailee()\nmy_email.sender("john.smith@fakeemail.com")\nmy_email.subject("Test email")\nmy_email.msgContent("some poorly formatted raw text")\nmy_email.sendTo(["jill.smith@fakeemail.com"])\nmy_email.server("smtp.fakeemail.com")\n\nif my_email.ready():\n    my_email.send()\n```\n\nSend multiple asyncronous emails\n\n```Python\nimport emailee\n\nemails_list = [{...}]\nserver_dict = {...}\n\nemails = emailee.AsyncThreads(emails_list, server_dict, outputFile=\'output.txt\')\n```\n\n## Documentation\n\nThere are only three classes exposed to the user of this library, `Emailee`, `AsyncThreads` and `AsyncMP`. The latter two utilise the Emailee class to send emails via the Threading or Multiprocessing async APIs respectively.\n\n### Emailee class\n\nIf you have used the `smtplib` and `email` libraries in the past, you may of found how difficult it is getting the email logic set up properly to handle attachments and different sending scenarios. All customisations are built into Emailee, which takes the guess work out of how to configure.\n\nThe following methods are exposed to the user:\n\n#### Emailee.sender(sender: str, replyTo: str = "") -> None\n\n* **sender** - email address of who is sending the email\n* **replyTo** (optional) - alternative email address the receiver can reply to\n\n#### Emailee.subject(subject: str = "") -> None\n\n* **subject** (optional) - email subject line, maximum of 255 characters\n\n#### Emailee.msgContent(msgText: str = "", msgHTML: str = "") -> None\n\n* **msgText** (optional) - email message in raw text format\n* **msgHTML** (optional) - email message in email HTML format\n\nIt\'s a good idea to use both of these to replicate your email, so users who can\'t read HTML emails can read your fallback raw text email. Look online for HTML email generators\n\n#### Emailee.sendTo(to: List[str] = [], cc: List[str] = [], bcc: List[str] = [], ignoreErrors: bool = False) -> None\n\n* **to** (optional) - list of to addresses to send emails to\n* **cc** (optional) - list of cc addresses to send emails to\n* **bcc** (optional) - list of bcc addresses to send emails to\n* **ignoreErrors** (optional) - all addresses are validated by regex to be in a valid email format. By default, if an email address is found to be invalid the program will raise an exception. If changed to True, the program will ignore and remove invalid emails and continue with sending the email.\n\n#### Emailee.attachmentFiles(attachmentFiles: List[str] = []) -> None\n\n* **attachmentFiles** (optional) - list of attachments for email, can be listed by relative or full path\n\n#### Emailee.server(smtpServer: str, port: int = 0, SSLTLS: str = "", authUsername: str = "", authPassword: str = "", timeout: int = 30) -> None\n\n* **smtpServer** - server name or IP address of SMTP server\n* **port** (optional) - port number for SMTP server, if connection not authenticated and port left blank, port will default to 25. If connection set to SSL or TLS and port left blank, port will default to 465 or 587 respectively.\n* **SSLTLS** (optional) - encrypted connection setting, value either "SSL" or "TLS"\n* **authUsername** (optional) - username for logging into the SMTP server, if authentication required. If left blank, but authPassword filled in, this will default to the sender email.\n* **authPassword** (optional) - password for logging into the SMTP server\n* **timeout** (optional) - server connection and email send timeout limit. Extend this value if on a slow network, or sending emails with large attachments\n\n#### Emailee.ready() -> bool\n\nReturns True if the minimal fields to send an email have been set.\n\n#### Email.send(outputFile: str) -> bool\n\n* **outputFile** (optional) - relative or full path location of a file to write report output metadata to, required for async classes when sending mail, file must be empty or not currently exist\n\nReturns True if successful, does not guarantee email was delivered, just sent.\n\n#### Email.__repr__() -> str\n\nPrint class function, will print a dictionary of email metadata, useful for testing and debugging.\n\n---\n\n### Data for async classes\n\nBoth async classes are very similar in their function and both require emails and server config in the exact same format:\n\n```Python\nmail_list = [\n    {\n        \'sender\': sender # see Emailee.sender()\n        \'replyTo\': replyTo # see Emailee.sender()\n        \'subject\': email_subject # see Emailee.subject()\n        \'msgText\': raw_text_message # see Emailee.msgContent()\n        \'msgHTML\': HTML_formatted_message # see Emailee.msgContent()\n        \'to\': list_of_to_emails # see Emailee.sendTo()\n        \'cc\': list_of_cc_emails # see Emailee.sendTo()\n        \'bcc\': list_of_bcc_emails # see Emailee.sendTo()\n        \'ignoreErrors\': True # see Emailee.sendTo()\n        \'attachmentFiles\': list_of_attachment_file_paths # see Emailee.attachmentFiles()\n    },\n    {...},\n]\n```\n\n```Python\nserver_dict = {\n    \'smtpServer\': SMTP_server # see Emailee.server()\n    \'port\': port_number # see Emailee.server()\n    \'SSLTLS\': \'TLS\' # see Emailee.server()\n    \'authUsername\': authenticated_username # see Emailee.server()\n    \'authPassword\': authenticated_password # see Emailee.server()\n}\n```\n\n### AsyncThreads class\n\nAsyncThreads uses the threading API to enable asyncronous sending of email. Threading only utilises the same CPU core that the Python program is currently running on, so it can only maximise a single core usage, but the benefits are you can throttle how many threads are allowed to run concurrently and it has a lower overhead compared to `AsyncMP`.\n\n#### AsyncThreads(mailList: List[Dict], serverDict: Dict, outputFile: str, maxThreads: int = 10, waitTime: float = 0)\n\n* **mailList** - see mail_list above for format\n* **serverDict** - see server_dict above for format\n* **outputFile** - relative or full path location of a file to write report output metadata to, file must be empty or not currently exist. Mandatory for async classes to handle errors.\n* **maxThreads** - maximum number of concurrent threads to send emails on\n* **waitTime** - wait time in seconds to pause between each thread being spawned, this helps if your SMTP server throttles your connection if you send too many emails in a short time span\n\n### AsyncMP class\n\nAsyncMP uses the multiprocessing API to enable asyncronous sending of email. Multiprocessing utilises all CPU cores of the system the Python program is running on, but does not allow throttling, so you run the risk of maxing out your CPU and RAM, slowing down your system.\n\n#### AsyncMP(mailList: List[Dict], serverDict: Dict, outputFile: str, waitTime: float = 0)\n\n* **mailList** - see mail_list above for format\n* **serverDict** - see server_dict above for format\n* **outputFile** - relative or full path location of a file to write report output metadata to, file must be empty or not currently exist. Mandatory for async classes to handle errors.\n* **waitTime** - wait time in seconds to pause between each process being spawned, this helps if your SMTP server throttles your connection if you send too many emails in a short time span\n\n### Reporting on async output\n\nUpon completion of either async class, you can call the `emailReport()` method to return a metadata list of all emails sent.\nThis won\'t work if an exception is thrown during the sending process, hence the **outputFile** requirement.\nThe output file will be written to after each successful email send so you can analyse against `emailReport()` or see who received emails before a thrown exception or manual cancellation of the sending process.\n\n## Development and Testing\n\nEmailee is built with [**poetry**](https://python-poetry.org/), tested with [**pytest**](https://pytest.org), [**tox**](https://tox.readthedocs.io/en/latest/) and [**coverage**](https://coverage.readthedocs.io/en/coverage-5.5/), type checked with [**mypy**](http://mypy-lang.org/) and formatted with [**black**](https://github.com/psf/black).\n\n### Dev Installation\n\n#### Install with Poetry\n\nClone the repo, `cd` into it and run `poetry install`\n\n#### Install with pip\n\nClone the repo, `cd` into it, build and run a new virtual environment, then open the `pyproject.toml` file and `pip install` all packages listed under **[tool.poetry.dev-dependencies]**\n\n### Testing\n\nTo successfully run local tests you will need to rename `tests/example.test.env.toml` to `tests/test.env.toml` and modify the config inside the file to contain valid email and SMTP server connection data.\n\nTesting and coverage can then be run with `pytest --cov-report term-missing --cov=emailee tests`\n\nTox can be used to run pytest against all support Python environments. Open `tox.ini` and check if you have all versions of Python installed listed under *envlist*.\n\n### Pre-commit hooks\n\nRun `pre-commit install` to install the pre-commit hooks in the `.pre-commit-config.yaml` file. Then run `pre-commit run --all-files` to auto-check every file for issues.\n\nFeel free to send me any changes or feedback.\n\n---\n\nThanks for reading :)\n\n:envelope: :envelope: :envelope: :envelope: :envelope:\n',
    'author': 'Matt Walsh',
    'author_email': 'git@mattwalsh.dev',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'http://github.com/mattwalshdev/emailee',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
